@@ -7,6 +7,7 @@
 # For further information on the license, see the LICENSE.txt file        #
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
+# pylint: disable=invalid-name,no-member
 """Delete the kombu tables that were used by the old Celery based daemon and the obsolete related timestamps
 
 Revision ID: f9a69de76a9a
@@ -17,7 +18,6 @@ Create Date: 2018-05-10 15:07:59.235950
 from alembic import op
 from sqlalchemy.sql import text
 
-
 # revision identifiers, used by Alembic.
 revision = 'f9a69de76a9a'
 down_revision = '6c629c886f84'
@@ -26,10 +26,12 @@ depends_on = None
 
 
 def upgrade():
+    """Migrations for the upgrade."""
     conn = op.get_bind()
 
     # Drop the kombu tables and delete the old timestamps and user related to the daemon in the DbSetting table
-    statement = text("""
+    statement = text(
+        """
             DROP TABLE IF EXISTS kombu_message;
             DROP TABLE IF EXISTS kombu_queue;
             DROP SEQUENCE IF EXISTS message_id_sequence;
@@ -41,9 +43,11 @@ def upgrade():
             DELETE FROM db_dbsetting WHERE key = 'daemon|task_start|updater';
             DELETE FROM db_dbsetting WHERE key = 'daemon|task_stop|submitter';
             DELETE FROM db_dbsetting WHERE key = 'daemon|task_start|submitter';
-    """)
+    """
+    )
     conn.execute(statement)
 
 
 def downgrade():
+    """Migrations for the downgrade."""
     print('There is no downgrade for the deletion of the kombu tables and the daemon timestamps')
