@@ -30,7 +30,7 @@ def edit_multiline_template(template_name, comment_marker='#=', extension=None,
 
     if content is not None:
         # Remove all comments, which are all lines that start with the comment marker
-        return re.sub(r'(^' + re.escape(comment_marker) + '.*$\n)+', '', content, flags=re.M).strip()
+        return re.sub(f'(^{re.escape(comment_marker)}.*$\n)+', '', content, flags=re.M).strip()
 
     return ''
 
