@@ -25,25 +25,23 @@ class CalculationNode(ProcessNode):
     _unstorable_message = 'storing for this node has been disabled'
 
     @property
-    def inputs(self):
+    def inputs(self) -> NodeLinksManager:
         """Return an instance of `NodeLinksManager` to manage incoming INPUT_CALC links
 
         The returned Manager allows you to easily explore the nodes connected to this node
         via an incoming INPUT_CALC link.
         The incoming nodes are reachable by their link labels which are attributes of the manager.
 
-        :return: `NodeLinksManager`
         """
         return NodeLinksManager(node=self, link_type=LinkType.INPUT_CALC, incoming=True)
 
     @property
-    def outputs(self):
+    def outputs(self) -> NodeLinksManager:
         """Return an instance of `NodeLinksManager` to manage outgoing CREATE links
 
         The returned Manager allows you to easily explore the nodes connected to this node
         via an outgoing CREATE link.
         The outgoing nodes are reachable by their link labels which are attributes of the manager.
 
-        :return: `NodeLinksManager`
         """
         return NodeLinksManager(node=self, link_type=LinkType.CREATE, incoming=False)
