@@ -139,13 +139,13 @@ class User(entities.Entity):
         :return: the user full name
         """
         if self.first_name and self.last_name:
-            full_name = '{} {} ({})'.format(self.first_name, self.last_name, self.email)
+            full_name = f'{self.first_name} {self.last_name} ({self.email})'
         elif self.first_name:
-            full_name = '{} ({})'.format(self.first_name, self.email)
+            full_name = f'{self.first_name} ({self.email})'
         elif self.last_name:
-            full_name = '{} ({})'.format(self.last_name, self.email)
+            full_name = f'{self.last_name} ({self.email})'
         else:
-            full_name = '{}'.format(self.email)
+            full_name = f'{self.email}'
 
         return full_name
 
