@@ -3,19 +3,13 @@
 # Copyright (c), The AiiDA team. All rights reserved.                     #
 # This file is part of the AiiDA code.                                    #
 #                                                                         #
-# The code is hosted on GitHub at https://github.com/aiidateam/aiida_core #
+# The code is hosted on GitHub at https://github.com/aiidateam/aiida-core #
 # For further information on the license, see the LICENSE.txt file        #
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
-from aiida.parsers.parser import Parser
+# pylint: disable=wildcard-import,undefined-variable
+"""Module for classes and utilities to write parsers for calculation jobs."""
 
+from .parser import *
 
-
-def ParserFactory(module):
-    """
-    Return a suitable Parser subclass.
-    """
-    from aiida.common.pluginloader import BaseFactory
-    from aiida.common.exceptions import MissingPluginError
-
-    return BaseFactory(module, Parser, 'aiida.parsers')
+__all__ = (parser.__all__)
